==> lkrg-0.2.tar.gz: lkrg-0.2/src/modules/exploit_detection/syscalls/p_call_usermodehelper/p_call_usermodehelper.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept *call_usermodehelper* function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When kernel calls user-mode helper, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.II.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_call_usermodehelper_kretprobe_state = 0x0;

static struct kretprobe p_call_usermodehelper_kretprobe = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,3,0)
    .kp.symbol_name = "____call_usermodehelper",
#else
    .kp.symbol_name = "call_usermodehelper_exec_async",
#endif
    .handler = p_call_usermodehelper_ret,
    .entry_handler = p_call_usermodehelper_entry,
    .data_size = sizeof(struct p_call_usermodehelper_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

int p_call_usermodehelper_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_entry>\n");

   p_ed_enforce_validation();

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);
   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_entry>\n");

   return 0;
}


int p_call_usermodehelper_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_ret>\n");

   // Update process
   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
      p_update_ed_process(p_tmp, current);
      p_set_ed_process_on(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_ret>\n");

   return 0x0;
}


int p_install_call_usermodehelper_hook(void) {

   int p_ret;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_call_usermodehelper_hook>\n");

   if ( (p_ret = register_kretprobe(&p_call_usermodehelper_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() failed! [err=%d]\n",p_ret);
      goto p_uninstall_call_usermodehelper_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
                     p_call_usermodehelper_kretprobe.kp.symbol_name, p_call_usermodehelper_kretprobe.kp.addr);
   p_call_usermodehelper_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_uninstall_call_usermodehelper_hook_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_call_usermodehelper_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_call_usermodehelper_hook(void) {

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_call_usermodehelper_hook>\n");

   if (!p_call_usermodehelper_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                     p_call_usermodehelper_kretprobe.kp.symbol_name ,p_call_usermodehelper_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_call_usermodehelper_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p\n",
                     p_call_usermodehelper_kretprobe.kp.symbol_name ,p_call_usermodehelper_kretprobe.kp.addr);
      p_call_usermodehelper_kretprobe_state = 0x0;
   }

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_call_usermodehelper_hook>\n");
}

==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/p_call_usermodehelper/p_call_usermodehelper.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept *call_usermodehelper* function
 *
 * Notes:
 *  - We are maintianing Red-Black tree of pid's for Exploit Detection feature.
 *    When kernel calls user-mode helper, we need to update RB tree.
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 12.II.2018
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_call_usermodehelper_kretprobe_state = 0x0;

static struct kretprobe p_call_usermodehelper_kretprobe = {
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,3,0)
    .kp.symbol_name = "____call_usermodehelper",
#else
    .kp.symbol_name = "call_usermodehelper_exec_async",
#endif
    .handler = p_call_usermodehelper_ret,
    .entry_handler = p_call_usermodehelper_entry,
    .data_size = sizeof(struct p_call_usermodehelper_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};

int p_call_usermodehelper_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_entry>\n");

   p_ed_enforce_validation();

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);
   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_entry>\n");

   return 0;
}


int p_call_usermodehelper_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

//   struct inode *p_inode;
   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_call_usermodehelper_ret>\n");

   // Update process
   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",task_pid_nr(current));
      p_update_ed_process(p_tmp, current);
      p_set_ed_process_on(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_call_usermodehelper_ret>\n");

   return 0x0;
}


int p_install_call_usermodehelper_hook(void) {

   int p_ret;

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_call_usermodehelper_hook>\n");

   if ( (p_ret = register_kretprobe(&p_call_usermodehelper_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_uninstall_call_usermodehelper_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_call_usermodehelper_kretprobe.kp.symbol_name,
               p_call_usermodehelper_kretprobe.kp.addr);
   p_call_usermodehelper_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_uninstall_call_usermodehelper_hook_out:

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_call_usermodehelper_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_call_usermodehelper_hook(void) {

   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_call_usermodehelper_hook>\n");

   if (!p_call_usermodehelper_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_call_usermodehelper_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_call_usermodehelper_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_call_usermodehelper_kretprobe.kp.symbol_name,
                  p_call_usermodehelper_kretprobe.kp.addr,
                  p_call_usermodehelper_kretprobe.nmissed);
      p_call_usermodehelper_kretprobe_state = 0x0;
   }

   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_call_usermodehelper_hook>\n");
}
